#!/bin/bash
#
# Use a single hooke domain with a constant unfolding rate, so
# unfolding force is proportional to time, and connect multiple
# identical Bell unfolding domains.  We expect an peaked death
# histogram.
#
# e.g.
#   spring constant k=df/dx    \__
#   velocity        v=dx/dt    /  `--> df/dt = kv --> f = kvt  (+ f(t=0))
#   unfolding rate  K= a e^(bf)   (frac/s)
#   let f(t=0) = 0
#   let P(f) be the population existing at force f (normed so P(t=f=0)=1).
#   the probability of dying (normed # deaths) between f and f+df is given by
#        dD = K P(f) * dt = K P(f) df/kv     since dt = df/kv
#   this is what we plot in the normalized histogram with dD in a bin of width
#   df.  The continuous analog is the function dD/df = K/kv P(f).
#
#   To find P(f), we need to solve
#        dP/dt = -KP
#        dP/P = -K dt = -a e^{bf} df/kv = -z e^{bf} df       where z ≡ a/kv
#        ln(P) = -z/b e^{bf) + c
#        P = C e^{-z/b e^{bf}}
#   Applying our f(t=0)=0 boundary condition we see that C = P(t=f=0) e^{z/b}.
#   Plugging P(f) into our formula for dD/df yields
#        dD/df = K/kv P(f) = z e^{bf}  P(t=f=0) e^{z/b} e^{-z/b e^{bf}}
#        dD/df = P(t=f=0) z e^{bf + z/b(1-e^{bf})}
#
#   The histogram scaling isn't dD/df, though, it's dD/d(bin)
#        dD/d(bin) = dD/df df/d(bin)
#                  = binwidth P(t=f=0) z e^{bf + z/b(1-e^{bf})}
#
# Reminders:
#   spring constant k=df/dx       (N/m)
#   velocity        v=dx/dt       (m/s)
#   unfolding rate  K= a e^(bf) = K0 e^(fdx/kBT)  (frac/s)
# usage: bell_rate.sh <num_domains> <v> <k> <K0> <dx> <T>

if [ $# -ne 6 ]
then
    echo "usage: $0 <num_domains> <v> <k> <K0> <dx> <T>"
    echo ""
    echo "where"
    echo "spring constant k=df/dx             (N/m)"
    echo "velocity        v=dx/dt             (m/s)"
    echo "unfolding rate  K= K0 e^(fdx/kBT)   (frac/s)"
    exit 1
fi

NDOM=$1
V=$2
SPRING=$3
K0=$4
DX=$5
T=$6
DEBUG=0

# since we're on the cluster, we'll go crazy and get really good statistics ;)
N=10000

SAWSIM=../../bin/sawsim
DATA=bell_rate_${NDOM}_${V}_${SPRING}_${K0}_${DX}_${T}.d
HIST=bell_rate_${NDOM}_${V}_${SPRING}_${K0}_${DX}_${T}.hist
COMPFILE=bell_rate_${NDOM}_${V}_${SPRING}_${K0}_${DX}_${T}.compare
GPFILE=bell_rate_${NDOM}_${V}_${SPRING}_${K0}_${DX}_${T}.gp
PNGFILE=bell_rate_${NDOM}_${V}_${SPRING}_${K0}_${DX}_${T}.png

# set up the theory
# our predicted histogram (see notes above) is
#   dD/d(bin) = binwidth P0 z e^{bf + z/b(1-e^{bf})}
# where z ≡ a/kv, and b = Δx/kBT
KB=1.3806503e-23 # J/K
B=`python -c "print ($DX/($KB*$T))"`
Z=`python -c "print (float($K0)/($SPRING*$V))"`
# find a good df (histogram bin width).
#    currently, scale based only on e^(bf) term in exponent.
df=`python -c "print (0.01/$B)"`

THEORY_FN="$df * $N * $NDOM * $Z * exp($B*x + $Z/$B *(1 - exp($B*x)))"
if [ $DEBUG -eq 1 ]; then
    echo "b = dx/kBT = $B"
    echo "z = K0/kv = $Z"
    echo "Theory --> P(x=f) = $THEORY_FN"
fi

# run the experiments
> $DATA
if [ "$ISACLUSTER" -eq 1 ]
then
    # Sawsim >= 0.6
    qcmd -w14:00:00 xtimes $N $SAWSIM -T$T -v$V -s cantilever,hooke,$SPRING -N1 \
	-s folded,null -N$NDOM -s unfolded,null \
        -k "'folded,unfolded,bell,{$K0,$DX}'" -q folded \
        | grep -v '^#' | cut -f1 >> $DATA
else
    # Sawsim >= 0.6
    xtimes $N $SAWSIM -T$T -v$V -s cantilever,hooke,$SPRING -N1 \
	-s folded,null -N$NDOM -s unfolded,null \
        -k "folded,unfolded,bell,{$K0,$DX}" -q folded \
        | grep -v '^#' | cut -f1 >> $DATA
fi

# histogram the data
if [ $DEBUG -eq 1 ]; then
    echo "cat $DATA | stem_leaf -v-1 -b$df -c | sed 's/://' > $HIST"
fi
cat $DATA | stem_leaf -v-1 -b$df -c | sed 's/://' > $HIST
wc -l "$HIST"
# fit the data to an exponential decay
if [ $DEBUG -eq 1 ]; then
    echo "cat $HIST | ./fit_bell"
fi
FIT=`cat $HIST | ./fit_bell` || exit 1
FITZ=`echo "$FIT" | sed -n 's/a:[[:space:]]*//p'`
FITB=`echo "$FIT" | sed -n 's/b:[[:space:]]*//p'`
echo -e "z = K0/kv:\t$FITZ\t(expected $Z)" > $COMPFILE
echo -e "b = dx/kBT:\t$FITB\t(expected $B)" >> $COMPFILE
cat $COMPFILE

# generate gnuplot script
if [ $DEBUG -eq 1 ]; then
    echo "generate Gnuplot script"
fi
echo "set terminal png" > $GPFILE
echo "set output '$PNGFILE'" >> $GPFILE
echo "set logscale y" >> $GPFILE
echo "set xlabel 'Force'" >> $GPFILE
echo "set ylabel 'Deaths'" >> $GPFILE
echo "theory(x) = $THEORY_FN" >> $GPFILE
echo "fit(x) = $df * $N *  $NDOM * $FITZ * exp($FITB*x + $FITZ/$FITB*(1-exp($FITB*x)))" >> $GPFILE
echo "plot  theory(x), '$HIST' with points, fit(x)" >> $GPFILE

if [ $DEBUG -eq 1 ]; then
    echo "Generate PNG"
fi
gnuplot $GPFILE
